/* -*-c-*-
 *
 * $Id$
 *
 * Main header file for TrIPE
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Trivial IP Encryption (TrIPE).
 *
 * TrIPE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * TrIPE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with TrIPE; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef TRIPE_H
#define TRIPE_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include "config.h"

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <pwd.h>
#include <grp.h>

#include <mLib/alloc.h>
#include <mLib/arena.h>
#include <mLib/base64.h>
#include <mLib/bres.h>
#include <mLib/dstr.h>
#include <mLib/env.h>
#include <mLib/fdflags.h>
#include <mLib/fwatch.h>
#include <mLib/mdwopt.h>
#include <mLib/quis.h>
#include <mLib/report.h>
#include <mLib/sel.h>
#include <mLib/selbuf.h>
#include <mLib/sig.h>
#include <mLib/str.h>
#include <mLib/sub.h>
#include <mLib/trace.h>
#include <mLib/tv.h>

#include <catacomb/buf.h>

#include <catacomb/gcipher.h>
#include <catacomb/gmac.h>
#include <catacomb/grand.h>
#include <catacomb/key.h>
#include <catacomb/paranoia.h>

#include <catacomb/noise.h>
#include <catacomb/rand.h>

#include <catacomb/mp.h>
#include <catacomb/mprand.h>
#include <catacomb/dh.h>
#include <catacomb/ec.h>
#include <catacomb/ec-keys.h>
#include <catacomb/group.h>

#include "tripe-protocol.h"
#include "util.h"

#undef sun

/*----- Magic numbers -----------------------------------------------------*/

/* --- Trace flags --- */

#define T_TUNNEL 1u
#define T_PEER 2u
#define T_PACKET 4u
#define T_ADMIN 8u
#define T_CRYPTO 16u
#define T_KEYSET 32u
#define T_KEYEXCH 64u
#define T_KEYMGMT 128u
#define T_CHAL 256u

#define T_ALL 511u

/* --- Units --- */

#define SEC(n) (n##u)
#define MIN(n) (n##u * 60u)
#define MEG(n) (n##ul * 1024ul * 1024ul)

/* --- Other things --- */

#define PKBUFSZ 65536

/*----- Cipher selections -------------------------------------------------*/

typedef struct algswitch {
  const gccipher *c;			/* Symmetric encryption scheme */
  const gccipher *mgf;			/* Mask-generation function */
  const gchash *h;			/* Hash function */
  const gcmac *m;			/* Message authentication code */
  size_t hashsz;			/* Hash output size */
  size_t tagsz;				/* Length to truncate MAC tags */
  size_t cksz, mksz;			/* Key lengths for @c@ and @m@ */
} algswitch;

extern algswitch algs;

#define MAXHASHSZ 64			/* Largest possible hash size */

#define HASH_STRING(h, s) GH_HASH((h), (s), sizeof(s))

/*----- Data structures ---------------------------------------------------*/

/* --- Socket addresses --- *
 *
 * A magic union of supported socket addresses.
 */

typedef union addr {
  struct sockaddr sa;
  struct sockaddr_in sin;
} addr;

/* --- Sequence number checking --- */

typedef struct seqwin {
  uint32 seq;				/* First acceptable input sequence */
  uint32 win;				/* Window of acceptable numbers */
} seqwin;

#define SEQ_WINSZ 32			/* Bits in sequence number window */

#define SEQ_RESET(iseq) { 

#define SEQ_OK 0			/* Sequence number valid */
#define SEQ_OLD -1			/* Sequence number too old */
#define SEQ_REPLAY -2			/* Definitely replayed */

/* --- A symmetric keyset --- *
 *
 * A keyset contains a set of symmetric keys for encrypting and decrypting
 * packets.  Keysets are stored in a list, sorted in reverse order of
 * creation, so that the most recent keyset (the one most likely to be used)
 * is first.
 *
 * Each keyset has a time limit and a data limit.  The keyset is destroyed
 * when either it has existed for too long, or it has been used to encrypt
 * too much data.  New key exchanges are triggered when keys are close to
 * expiry.
 */

typedef struct keyset {
  struct keyset *next;			/* Next active keyset in the list */
  unsigned ref;				/* Reference count for keyset */
  struct peer *p;			/* Pointer to peer structure */
  time_t t_exp;				/* Expiry time for this keyset */
  unsigned long sz_exp;			/* Data limit for the keyset */
  T( unsigned seq; )			/* Sequence number for tracing */
  unsigned f;				/* Various useful flags */
  gcipher *cin, *cout; 			/* Keyset ciphers for encryption */
  size_t tagsz;				/* Length to truncate MAC tags */
  gmac *min, *mout;	       		/* Keyset MACs for integrity */
  uint32 oseq;				/* Outbound sequence number */
  seqwin iseq;				/* Inbound sequence number */
} keyset;

#define KSF_LISTEN 1u			/* Don't encrypt packets yet */
#define KSF_LINK 2u			/* Key is in a linked list */

/* --- Key exchange --- *
 *
 * TrIPE uses the Wrestlers Protocol for its key exchange.  The Wrestlers
 * Protocol has a number of desirable features (e.g., perfect forward
 * secrecy, and zero-knowledge authentication) which make it attractive for
 * use in TrIPE.  The Wrestlers Protocol was designed by Mark Wooding and
 * Clive Jones.
 */

#define KX_NCHAL 16u
#define KX_THRESH 4u

typedef struct kxchal {
  struct keyexch *kx;			/* Pointer back to key exchange */
  ge *c;				/* Responder's challenge */
  ge *r;				/* My reply to the challenge */
  keyset *ks;				/* Pointer to temporary keyset */
  unsigned f;				/* Various useful flags */
  sel_timer t;				/* Response timer for challenge */
  octet hc[MAXHASHSZ];			/* Hash of his challenge */
  mp *ck;				/* The check value */
  octet hswrq_in[MAXHASHSZ];		/* Inbound switch request message */
  octet hswok_in[MAXHASHSZ];		/* Inbound switch confirmation */
  octet hswrq_out[MAXHASHSZ];		/* Outbound switch request message */
  octet hswok_out[MAXHASHSZ];		/* Outbound switch confirmation */
} kxchal;

typedef struct keyexch {
  struct peer *p;			/* Pointer back to the peer */
  keyset **ks;				/* Peer's list of keysets */
  unsigned f;				/* Various useful flags */
  unsigned s;				/* Current state in exchange */
  sel_timer t;				/* Timer for next exchange */
  ge *kpub;				/* Peer's public key */
  time_t texp_kpub;			/* Expiry time for public key */
  mp *alpha;				/* My temporary secret */
  ge *c;				/* My challenge */
  ge *rx;				/* The expected response */
  unsigned nr;				/* Number of extant responses */
  time_t t_valid;			/* When this exchange goes bad */
  octet hc[MAXHASHSZ];			/* Hash of my challenge */
  kxchal *r[KX_NCHAL];			/* Array of challenges */
} keyexch;

#define KXF_TIMER 1u			/* Waiting for a timer to go off */
#define KXF_DEAD 2u			/* The key-exchanger isn't up */
#define KXF_PUBKEY 4u			/* Key exchanger has a public key */

enum {
  KXS_DEAD,				/* Uninitialized state (magical) */
  KXS_CHAL,				/* Main answer-challenges state */
  KXS_COMMIT,				/* Committed: send switch request */
  KXS_SWITCH				/* Switched: send confirmation */
};

/* --- Tunnel structure --- *
 *
 * Used to maintain system-specific information about the tunnel interface.
 */

typedef struct tunnel tunnel;
struct peer;

typedef struct tunnel_ops {
  const char *name;			/* Name of this tunnel driver */
  void (*init)(void);			/* Initializes the system */
  tunnel *(*create)(struct peer */*p*/); /* Initializes a new tunnel */
  const char *(*ifname)(tunnel */*t*/);	/* Returns tunnel's interface name */
  void (*inject)(tunnel */*t*/, buf */*b*/); /* Sends packet through if */
  void (*destroy)(tunnel */*t*/);	/* Destroys a tunnel */
} tunnel_ops;

#ifndef TUN_INTERNALS
struct tunnel { const tunnel_ops *ops; };
#endif

/* --- Peer statistics --- *
 *
 * Contains various interesting and not-so-interesting statistics about a
 * peer.  This is updated by various parts of the code.  The format of the
 * structure isn't considered private, and @p_stats@ returns a pointer to the
 * statistics block for a given peer.
 */

typedef struct stats {
  unsigned long sz_in, sz_out;		/* Size of all data in and out */
  unsigned long sz_kxin, sz_kxout;	/* Size of key exchange messages */
  unsigned long sz_ipin, sz_ipout;	/* Size of encapsulated IP packets */
  time_t t_start, t_last, t_kx;		/* Time peer created, last pk, kx */
  unsigned long n_reject;		/* Number of rejected packets */
  unsigned long n_in, n_out;		/* Number of packets in and out */
  unsigned long n_kxin, n_kxout;	/* Number of key exchange packets */
  unsigned long n_ipin, n_ipout;	/* Number of encrypted packets */
} stats;

/* --- Peer structure --- *
 *
 * The main structure which glues everything else together.
 */

typedef struct peerspec {
  char *name;				/* Peer's name */
  const tunnel_ops *tops;		/* Tunnel operations */
  unsigned long t_ka;			/* Keep alive interval */
  addr sa;				/* Socket address to speak to */
  size_t sasz;				/* Socket address size */
} peerspec;

typedef struct peer {
  struct peer *next, *prev;		/* Links to next and previous */
  struct ping *pings;			/* Pings we're waiting for */
  peerspec spec;			/* Specifications for this peer */
  tunnel *t;				/* Tunnel for local packets */
  keyset *ks;				/* List head for keysets */
  buf b;				/* Buffer for sending packets */
  stats st;				/* Statistics */
  keyexch kx;				/* Key exchange protocol block */
  sel_timer tka;			/* Timer for keepalives */
} peer;

typedef struct ping {
  struct ping *next, *prev;		/* Links to next and previous */
  peer *p;				/* Peer so we can free it */
  unsigned msg;				/* Kind of response expected */
  uint32 id;				/* Id so we can recognize response */
  octet magic[32];			/* Some random data */
  sel_timer t;				/* Timeout for ping */
  void (*func)(int /*rc*/, void */*arg*/); /* Function to call when done */
  void *arg;				/* Argument for callback */
} ping;

enum {
  PING_NONOTIFY = -1,
  PING_OK = 0,
  PING_TIMEOUT,
  PING_PEERDIED,
  PING_MAX
};

/* --- Admin structure --- */

#define OBUFSZ 16384u

typedef struct obuf {
  struct obuf *next;			/* Next buffer in list */
  char *p_in, *p_out;			/* Pointers into the buffer */
  char buf[OBUFSZ];			/* The actual buffer */
} obuf;

typedef struct oqueue {
  obuf *hd, *tl;			/* Head and tail pointers */
} oqueue;

struct admin;

typedef struct admin_bgop {
  struct admin_bgop *next, *prev;	/* Links to next and previous */
  struct admin *a;			/* Owner job */
  char *tag;				/* Tag string for messages */
  void (*cancel)(struct admin_bgop *);	/* Destructor function */
} admin_bgop;

typedef struct admin_resop {
  admin_bgop bg;			/* Background operation header */
  char *addr;				/* Hostname to be resolved */
  bres_client r;			/* Background resolver task */
  sel_timer t;				/* Timer for resolver */
  addr sa;				/* Socket address */
  size_t sasz;				/* Socket address size */
  void (*func)(struct admin_resop *, int); /* Handler */
} admin_resop;

enum { ARES_OK, ARES_FAIL };

typedef struct admin_addop {
  admin_resop r;			/* Name resolution header */
  peerspec peer;			/* Peer pending creation */
} admin_addop;

typedef struct admin_greetop {
  admin_resop r;			/* Name resolution header */
  void *c;				/* Challenge block */
  size_t sz;				/* Length of challenge */
} admin_greetop;

typedef struct admin_pingop {
  admin_bgop bg;			/* Background operation header */
  ping ping;				/* Ping pending response */
  struct timeval pingtime;		/* Time last ping was sent */
} admin_pingop;  

typedef struct admin {
  struct admin *next, *prev;		/* Links to next and previous */
  unsigned f;				/* Various useful flags */
  unsigned ref;				/* Reference counter */
#ifndef NTRACE
  unsigned seq;				/* Sequence number for tracing */
#endif
  oqueue out;				/* Output buffer list */
  oqueue delay;				/* Delayed output buffer list */
  admin_bgop *bg;			/* Backgrounded operations */
  selbuf b;				/* Line buffer for commands */
  sel_file w;				/* Selector for write buffering */
} admin;

#define AF_DEAD 1u			/* Destroy this admin block */
#define AF_CLOSE 2u			/* Client closed connection */
#define AF_NOTE 4u			/* Catch notifications */
#define AF_WARN 8u			/* Catch warning messages */
#ifndef NTRACE
  #define AF_TRACE 16u			/* Catch tracing */
#endif

#ifndef NTRACE
#  define AF_ALLMSGS (AF_NOTE | AF_TRACE | AF_WARN)
#else
#  define AF_ALLMSGS (AF_NOTE | AF_WARN)
#endif

/*----- Global variables --------------------------------------------------*/

extern sel_state sel;			/* Global I/O event state */
extern group *gg;			/* The group we work in */
extern mp *kpriv;			/* Our private key */
extern octet buf_i[PKBUFSZ], buf_o[PKBUFSZ], buf_t[PKBUFSZ];
extern const tunnel_ops *tunnels[];	/* Table of tunnels (0-term) */
extern const tunnel_ops *tun_default;	/* Default tunnel to use */

#ifndef NTRACE
extern const trace_opt tr_opts[];	/* Trace options array */
extern unsigned tr_flags;		/* Trace options flags */
#endif

/*----- Other macros ------------------------------------------------------*/

#define TIMER noise_timer(RAND_GLOBAL)

/*----- Key management ----------------------------------------------------*/

/* --- @km_reload@ --- *
 *
 * Arguments:	---
 *
 * Returns:	Zero if OK, nonzero to force reloading of keys.
 *
 * Use:		Checks the keyrings to see if they need reloading.
 */

extern int km_reload(void);

/* --- @km_init@ --- *
 *
 * Arguments:	@const char *kr_priv@ = private keyring file
 *		@const char *kr_pub@ = public keyring file
 *		@const char *tag@ = tag to load
 *
 * Returns:	---
 *
 * Use:		Initializes, and loads the private key.
 */

extern void km_init(const char */*kr_priv*/, const char */*kr_pub*/,
		    const char */*tag*/);

/* --- @km_getpubkey@ --- *
 *
 * Arguments:	@const char *tag@ = public key tag to load
 *		@ge *kpub@ = where to put the public key
 *		@time_t *t_exp@ = where to put the expiry time
 *
 * Returns:	Zero if OK, nonzero if it failed.
 *
 * Use:		Fetches a public key from the keyring.
 */

extern int km_getpubkey(const char */*tag*/, ge */*kpub*/,
			time_t */*t_exp*/);

/*----- Key exchange ------------------------------------------------------*/

/* --- @kx_start@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *		@int forcep@ = nonzero to ignore the quiet timer
 *
 * Returns:	---
 *
 * Use:		Stimulates a key exchange.  If a key exchage is in progress,
 *		a new challenge is sent (unless the quiet timer forbids
 *		this); if no exchange is in progress, one is commenced.
 */

extern void kx_start(keyexch */*kx*/, int /*forcep*/);

/* --- @kx_message@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *		@unsigned msg@ = the message code
 *		@buf *b@ = pointer to buffer containing the packet
 *
 * Returns:	---
 *
 * Use:		Reads a packet containing key exchange messages and handles
 *		it.
 */

extern void kx_message(keyexch */*kx*/, unsigned /*msg*/, buf */*b*/);

/* --- @kx_free@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *
 * Returns:	---
 *
 * Use:		Frees everything in a key exchange context.
 */

extern void kx_free(keyexch */*kx*/);

/* --- @kx_newkeys@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *
 * Returns:	---
 *
 * Use:		Informs the key exchange module that its keys may have
 *		changed.  If fetching the new keys fails, the peer will be
 *		destroyed, we log messages and struggle along with the old
 *		keys.
 */

extern void kx_newkeys(keyexch */*kx*/);

/* --- @kx_init@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *		@peer *p@ = pointer to peer context
 *		@keyset **ks@ = pointer to keyset list
 *
 * Returns:	Zero if OK, nonzero if it failed.
 *
 * Use:		Initializes a key exchange module.  The module currently
 *		contains no keys, and will attempt to initiate a key
 *		exchange.
 */

extern int kx_init(keyexch */*kx*/, peer */*p*/, keyset **/*ks*/);

/*----- Keysets and symmetric cryptography --------------------------------*/

/* --- @ks_drop@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Decrements a keyset's reference counter.  If the counter hits
 *		zero, the keyset is freed.
 */

extern void ks_drop(keyset */*ks*/);

/* --- @ks_gen@ --- *
 *
 * Arguments:	@const void *k@ = pointer to key material
 *		@size_t x, y, z@ = offsets into key material (see below)
 *		@peer *p@ = pointer to peer information
 *
 * Returns:	A pointer to the new keyset.
 *
 * Use:		Derives a new keyset from the given key material.  The
 *		offsets @x@, @y@ and @z@ separate the key material into three
 *		parts.  Between the @k@ and @k + x@ is `my' contribution to
 *		the key material; between @k + x@ and @k + y@ is `your'
 *		contribution; and between @k + y@ and @k + z@ is a shared
 *		value we made together.  These are used to construct two
 *		pairs of symmetric keys.  Each pair consists of an encryption
 *		key and a message authentication key.  One pair is used for
 *		outgoing messages, the other for incoming messages.
 *
 *		The new key is marked so that it won't be selected for output
 *		by @ksl_encrypt@.  You can still encrypt data with it by
 *		calling @ks_encrypt@ directly.
 */

extern keyset *ks_gen(const void */*k*/,
		      size_t /*x*/, size_t /*y*/, size_t /*z*/,
		      peer */*p*/);

/* --- @ks_tregen@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *
 * Returns:	The time at which moves ought to be made to replace this key.
 */

extern time_t ks_tregen(keyset */*ks*/);

/* --- @ks_activate@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Activates a keyset, so that it can be used for encrypting
 *		outgoing messages.
 */

extern void ks_activate(keyset */*ks*/);

/* --- @ks_encrypt@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *		@unsigned ty@ = message type
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Zero if OK, nonzero if the key needs replacing.  If the
 *		encryption failed, the output buffer is broken and zero is
 *		returned.
 *
 * Use:		Encrypts a block of data using the key.  Note that the `key
 *		ought to be replaced' notification is only ever given once
 *		for each key.  Also note that this call forces a keyset to be
 *		used even if it's marked as not for data output.
 */

extern int ks_encrypt(keyset */*ks*/, unsigned /*ty*/,
		      buf */*b*/, buf */*bb*/);

/* --- @ks_decrypt@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *		@unsigned ty@ = expected type code
 *		@buf *b@ = pointer to an input buffer
 *		@buf *bb@ = pointer to an output buffer
 *
 * Returns:	Zero on success, or nonzero if there was some problem.
 *
 * Use:		Attempts to decrypt a message using a given key.  Note that
 *		requesting decryption with a key directly won't clear a
 *		marking that it's not for encryption.
 */

extern int ks_decrypt(keyset */*ks*/, unsigned /*ty*/,
		      buf */*b*/, buf */*bb*/);

/* --- @ksl_free@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *
 * Returns:	---
 *
 * Use:		Frees (releases references to) all of the keys in a keyset.
 */

extern void ksl_free(keyset **/*ksroot*/);

/* --- @ksl_link@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Links a keyset into a list.  A keyset can only be on one list
 *		at a time.  Bad things happen otherwise.
 */

extern void ksl_link(keyset **/*ksroot*/, keyset */*ks*/);

/* --- @ksl_prune@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *
 * Returns:	---
 *
 * Use:		Prunes the keyset list by removing keys which mustn't be used
 *		any more.
 */

extern void ksl_prune(keyset **/*ksroot*/);

/* --- @ksl_encrypt@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@unsigned ty@ = message type
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Nonzero if a new key is needed.
 *
 * Use:		Encrypts a packet.
 */

extern int ksl_encrypt(keyset **/*ksroot*/, unsigned /*ty*/,
		       buf */*b*/, buf */*bb*/);

/* --- @ksl_decrypt@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@unsigned ty@ = expected type code
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Nonzero if the packet couldn't be decrypted.
 *
 * Use:		Decrypts a packet.
 */

extern int ksl_decrypt(keyset **/*ksroot*/, unsigned /*ty*/,
		       buf */*b*/, buf */*bb*/);

/*----- Challenges --------------------------------------------------------*/

/* --- @c_new@ --- *
 *
 * Arguments:	@buf *b@ = where to put the challenge
 *
 * Returns:	Zero if OK, nonzero on error.
 *
 * Use:		Issues a new challenge.
 */

extern int c_new(buf */*b*/);

/* --- @c_check@ --- *
 *
 * Arguments:	@buf *b@ = where to find the challenge
 *
 * Returns:	Zero if OK, nonzero if it didn't work.
 *
 * Use:		Checks a challenge.  On failure, the buffer is broken.
 */

extern int c_check(buf */*b*/);

/*----- Administration interface ------------------------------------------*/

/* --- @a_warn@ --- *
 *
 * Arguments:	@const char *fmt@ = pointer to format string
 *		@...@ = other arguments
 *
 * Returns:	---
 *
 * Use:		Informs all admin connections of a warning.
 */

extern void a_warn(const char */*fmt*/, ...);

/* --- @a_notify@ --- *
 *
 * Arguments:	@const char *fmt@ = pointer to format string
 *		@...@ = other arguments
 *
 * Returns:	---
 *
 * Use:		Sends a notification to interested admin connections.
 */

extern void a_notify(const char */*fmt*/, ...);

/* --- @a_create@ --- *
 *
 * Arguments:	@int fd_in, fd_out@ = file descriptors to use
 *		@unsigned f@ = initial flags to set
 *
 * Returns:	---
 *
 * Use:		Creates a new admin connection.
 */

extern void a_create(int /*fd_in*/, int /*fd_out*/, unsigned /*f*/);

/* --- @a_quit@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Shuts things down nicely.
 */

extern void a_quit(void);

/* --- @a_daemon@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Informs the admin module that it's a daemon.
 */

extern void a_daemon(void);

/* --- @a_init@ --- *
 *
 * Arguments:	@const char *sock@ = socket name to create
 *
 * Returns:	---
 *
 * Use:		Creates the admin listening socket.
 */

extern void a_init(const char */*sock*/);

/*----- Peer management ---------------------------------------------------*/

/* --- @p_txstart@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *		@unsigned msg@ = message type code
 *
 * Returns:	A pointer to a buffer to write to.
 *
 * Use:		Starts sending to a peer.  Only one send can happen at a
 *		time.
 */

extern buf *p_txstart(peer */*p*/, unsigned /*msg*/);

/* --- @p_txend@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *
 * Returns:	---
 *
 * Use:		Sends a packet to the peer.
 */

extern void p_txend(peer */*p*/);

/* --- @p_pingsend@ --- *
 *
 * Arguments:	@peer *p@ = destination peer
 *		@ping *pg@ = structure to fill in
 *		@unsigned type@ = message type
 *		@unsigned long timeout@ = how long to wait before giving up
 *		@void (*func)(int, void *)@ = callback function
 *		@void *arg@ = argument for callback
 *
 * Returns:	Zero if successful, nonzero if it failed.
 *
 * Use:		Sends a ping to a peer.  Call @func@ with a nonzero argument
 *		if we get an answer within the timeout, or zero if no answer.
 */

extern int p_pingsend(peer */*p*/, ping */*pg*/, unsigned /*type*/,
		      unsigned long /*timeout*/,
		      void (*/*func*/)(int, void *), void */*arg*/);

/* --- @p_pingdone@ --- *
 *
 * Arguments:	@ping *p@ = ping structure
 *		@int rc@ = return code to pass on
 *
 * Returns:	---
 *
 * Use:		Disposes of a ping structure, maybe sending a notification.
 */

extern void p_pingdone(ping */*p*/, int /*rc*/);

/* --- @p_greet@ --- *
 *
 * Arguments:	@peer *p@ = peer to send to
 *		@const void *c@ = pointer to challenge
 *		@size_t sz@ = size of challenge
 *
 * Returns:	---
 *
 * Use:		Sends a greeting packet.
 */

extern void p_greet(peer */*p*/, const void */*c*/, size_t /*sz*/);

/* --- @p_tun@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *		@buf *b@ = buffer containing incoming packet
 *
 * Returns:	---
 *
 * Use:		Handles a packet which needs to be sent to a peer.
 */

extern void p_tun(peer */*p*/, buf */*b*/);

/* --- @p_keyreload@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Forces a check of the daemon's keyring files.
 */

extern void p_keyreload(void);

/* --- @p_interval@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Called periodically to do tidying.
 */

extern void p_interval(void);

/* --- @p_stats@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's statistics.
 */

extern stats *p_stats(peer */*p*/);

/* --- @p_ifname@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's interface name.
 */

extern const char *p_ifname(peer */*p*/);

/* --- @p_addr@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's address.
 */

extern const addr *p_addr(peer */*p*/);

/* --- @p_init@ --- *
 *
 * Arguments:	@struct in_addr addr@ = address to bind to
 *		@unsigned port@ = port number to listen to
 *
 * Returns:	---
 *
 * Use:		Initializes the peer system; creates the socket.
 */

extern void p_init(struct in_addr /*addr*/, unsigned /*port*/);

/* --- @p_port@ --- *
 *
 * Arguments:	---
 *
 * Returns:	Port number used for socket.
 */

unsigned p_port(void);

/* --- @p_create@ --- *
 *
 * Arguments:	@peerspec *spec@ = information about this peer
 *
 * Returns:	Pointer to the peer block, or null if it failed.
 *
 * Use:		Creates a new named peer block.  No peer is actually attached
 *		by this point.
 */

extern peer *p_create(peerspec */*spec*/);

/* --- @p_name@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's name.
 *
 * Use:		Equivalent to @p_spec(p)->name@.
 */

extern const char *p_name(peer */*p*/);

/* --- @p_spec@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	Pointer to the peer's specification
 */

extern const peerspec *p_spec(peer */*p*/);

/* --- @p_find@ --- *
 *
 * Arguments:	@const char *name@ = name to look up
 *
 * Returns:	Pointer to the peer block, or null if not found.
 *
 * Use:		Finds a peer by name.
 */

extern peer *p_find(const char */*name*/);

/* --- @p_destroy@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer
 *
 * Returns:	---
 *
 * Use:		Destroys a peer.
 */

extern void p_destroy(peer */*p*/);

/* --- @p_first@, @p_next@ --- *
 *
 * Arguments:	@peer *p@ = a peer block
 *
 * Returns:	@peer_first@ returns the first peer in some ordering;
 *		@peer_next@ returns the peer following a given one in the
 *		same ordering.  Null is returned for the end of the list.
 */

extern peer *p_first(void);
extern peer *p_next(peer */*p*/);

/*----- Tunnel drivers ----------------------------------------------------*/

#ifdef TUN_LINUX
  extern const tunnel_ops tun_linux;
#endif

#ifdef TUN_UNET
  extern const tunnel_ops tun_unet;
#endif

#ifdef TUN_BSD
  extern const tunnel_ops tun_bsd;
#endif

extern const tunnel_ops tun_slip;

/*----- Other handy utilities ---------------------------------------------*/

/* --- @mpstr@ --- *
 *
 * Arguments:	@mp *m@ = a multiprecision integer
 *
 * Returns:	A pointer to the integer's textual representation.
 *
 * Use:		Converts a multiprecision integer to a string.  Corrupts
 *		@buf_t@.
 */

extern const char *mpstr(mp */*m*/);

/* --- @gestr@ --- *
 *
 * Arguments:	@group *g@ = a group
 *		@ge *x@ = a group element
 *
 * Returns:	A pointer to the element's textual representation.
 *
 * Use:		Converts a group element to a string.  Corrupts
 *		@buf_t@.
 */

extern const char *gestr(group */*g*/, ge */*x*/);

/* --- @timestr@ --- *
 *
 * Arguments:	@time_t t@ = a time to convert
 *
 * Returns:	A pointer to a textual representation of the time.
 *
 * Use:		Converts a time to a textual representation.  Corrupts
 *		@buf_t@.
 */

extern const char *timestr(time_t /*t*/);

/* --- @mystrieq@ --- *
 *
 * Arguments:	@const char *x, *y@ = two strings
 *
 * Returns:	True if @x@ and @y are equal, up to case.
 */

extern int mystrieq(const char */*x*/, const char */*y*/);

/* --- @b64_encode@ --- *
 *
 * Arguments:	@const void *p@ = pointer to some gorp
 *		@size_t sz@ = size of the gorp
 *
 * Returns:	Pointer to base64-encoded version in @buf_t@.
 */

extern const char *b64_encode(const void */*p*/, size_t /*sz*/);

/* --- @seq_reset@ --- *
 *
 * Arguments:	@seqwin *s@ = sequence-checking window
 *
 * Returns:	---
 *
 * Use:		Resets a sequence number window.
 */

extern void seq_reset(seqwin */*s*/);

/* --- @seq_check@ --- *
 *
 * Arguments:	@seqwin *s@ = sequence-checking window
 *		@uint32 q@ = sequence number to check
 *
 * Returns:	A @SEQ_@ code.
 *
 * Use:		Checks a sequence number against the window, updating things
 *		as necessary.
 */

extern int seq_check(seqwin */*s*/, uint32 /*q*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
